import urllib
import httplib as httpslib
import string

APPROVED = "approved"
DENIED = "denied"
ERROR = "error"

from zLOG import LOG,INFO,DEBUG


class AuthorizeNetPayment:
    """
    a Payment class for talking to Authorize.net servers
    """
    
    def __init__(self, **kwargs):
        self.card = "" # card number
        self.merchant = "" # merchant's ID
        self.tran_key = "" # merchant's Transaction Key
        self.expires = "" #
        self.first_name = ""
        self.last_name = ""
        self.address = ""
        self.city = ""
        self.state = ""
        self.zip = ""
        self.country = ""
        self.ship_first_name = ""
        self.ship_last_name = ""
        self.ship_address = ""
        self.ship_city = ""
        self.ship_state = ""
        self.ship_country = ""
        self.ship_zip = ""
        self.phone = ""
        self.fax = ""
        self.orgtype = "I"
        self.customer_ip = ""
        self.version = ""
        self.test = ""
        self.email_customer = ""
        self.invoice_number = ""

        for key in kwargs.keys():
            setattr(self, key, kwargs[key])

        self._secureServer = "secure.authorize.net"
        self._securePage = "gateway/transact.dll"

    def _submit(self, dict, page=None, headers=[]):
        """
        Given a dict of form variables, make the HTTPS POST.
        """
        data = urllib.urlencode(dict)

        if page:
            relurl = page
        else:
            relurl = self._securePage

        h = httpslib.HTTPSConnection(self._secureServer)
        h.putrequest('POST', "/" + relurl)
        h.putheader('Content-type', 'application/x-www-form-urlencoded')
        h.putheader('Content-length', '%d' % len(data))
        for head in headers:
            h.putheader(*head)
        h.endheaders()
        h.send(data)

        response = h.getresponse()
        content = response.read()
        """fp = h.getfile()

        content = ""
        chunk = "start"
        while len(chunk) != 0:
            chunk = fp.read()
            content += chunk
        fp.close()
        """
        
        return content



    def charge(self, amount, description=""):
        """Charge the card..."""
        
        values = {

            ## first the AuthNet-required stuff:
            "x_version": self.version,
            "x_delim_data": "TRUE",
            "x_relay_response": "FALSE",
            "x_tran_key": self.tran_key,
            "x_test_request":self.test,
            
            ## the IP address of the customer
            "x_customer_ip":self.customer_ip,
            
            "x_email_customer":self.email_customer,
            
            ## transaction details
            "x_first_name": self.first_name,
            "x_last_name": self.last_name,
            "x_company": self.company,
            "x_address": self.address,
            "x_city": self.city,
            "x_state": self.state,
            "x_zip": self.zip,
            "x_country": self.country,
            "x_ship_to_first_name": self.ship_first_name,
            "x_ship_to_last_name": self.ship_last_name,
            "x_ship_to_address": self.ship_address,
            "x_ship_to_city": self.ship_city,
            "x_ship_to_state": self.ship_state,
            "x_ship_to_zip": self.ship_zip,
            "x_ship_to_country": self.ship_country,
            "x_email": self.email,
            "x_phone": self.phone,
            "x_last_name": self.last_name,
            "x_customer_organization_type": self.orgtype,
            
            ## then the Payment stuff:
            "x_login": self.merchant,
            "x_card_num": self.card,
            "x_exp_date": self.expires,

            ## finally, the amount:
            "x_amount": amount,
            
            # invoice number and descrip
            "x_invoice_num": self.invoice_number,
            "x_description": description,
            }

        content = self._submit(values)
        
        # now parse the results
        (responseCode,
         responseSubCode,
         responseReasonCode,
         responseReasonText,
         authCode,
         avsCode,
         transID) = string.split(content, ",")[:7]
        
        # XXX log the response information
        order_id = self.invoice_number
        LOG('AuthorizeDotNet', INFO,"\n*******\norder #%(order_id)s\nResponse Code: %(responseCode)s\nResponse SubCode: %(responseSubCode)s\nResponseResonCode: %(responseReasonCode)s\nResponse reason text: %(responseReasonText)s\nAuth code: %(authCode)s\nAVS code: %(avsCode)s\nTrans ID: %(transID)s\n*******\n" % locals())
        
        if responseCode == "1":
            self.result = APPROVED
            self.error = None
        elif responseCode == "2":
            self.result = DENIED
            self.error = responseReasonText
        else:
            self.result = ERROR
            self.error = responseReasonText
            
